/*
  gpsdialog.h
  Copyright (C) 2008-2009 Tomohiro Masubuchi <bucchi@e-ml.jp>

  This file is part of qgmap.

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __GPSDIALOG_H__
#define __GPSDIALOG_H__

#include "qgmap.h"
#include <qdialog.h>
#include <qstring.h>
class QLabel;
class QLineEdit;
class QSpinBox;
class QButtonGroup;
class QRadioButton;
class QComboBox;

class GpsDialog : public QDialog
{
	Q_OBJECT

  public:
	GpsDialog( QWidget* parent = 0, const char* name = 0, bool modal = TRUE, WFlags fl = 0 );
	~GpsDialog();

	void loadConfig();
	void saveConfig();

  protected:
	QComboBox *selecter;
	QLineEdit *name;
	QLineEdit *device;
	QLineEdit *preCommand;
	QLineEdit *postCommand;
	QSpinBox *interval;
	QRadioButton *tokyo_datum;
	QRadioButton *wgs84_datum;
	QComboBox *tz;

	struct conf {
		QString name;
		QString device;
		QString preCommand;
		QString postCommand;
		int interval;
		Datum datum;
		int timezone;
	} configs[MAXGPSSETTINGS];

	int now_index;

	void setInterval(int interval);
	void setDatum(Datum datum);
	void setTimeZone(int timezone);

	int getInterval(void);
	Datum getDatum(void);
	int getTimeZone(void);

	void storeItems(int index);
	void loadItems(int index);

  private slots:
	void changeName(const QString &str);
	void changeDevice(int index);

  public slots:
	void accept();
};

#endif // __GPSDIALOG_H__
