/*
  mappathdialog.cpp
  Copyright (C) 2008-2009 Tomohiro Masubuchi <bucchi@e-ml.jp>

  This file is part of qgmap.

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; If not, see <http://www.gnu.org/licenses/>.
*/

#include "qgmap.h"
#include "mappathdialog.h"
#include "mainwindow.h"
#include "filedialog.h"
#include <qlayout.h>
#include <qlabel.h>
#include <qstring.h>
#include <qhbox.h>
#include <qvbox.h>
#include <qmessagebox.h>

#include <qlistview.h>
#include <qpushbutton.h>
#include <qfileinfo.h>

#define DIALOG_WIDTH  450
#define DIALOG_HEIGHT 300

#define BUTTON_WIDTH 30

MapPathDialog::MapPathDialog( QWidget* parent, const char* name,
						  bool modal, WFlags fl)
	: QDialog( parent, name, modal, fl )
{
	resize( QSize(DIALOG_WIDTH, DIALOG_HEIGHT).expandedTo(minimumSizeHint()) );

	setCaption(QString::fromUtf8("地図のパス設定"));
#ifdef QWS
	setFont(QFont("lcfont", 14));
#else
	setFont(QFont("gothic", 8));
#endif

	QVBoxLayout *base_vbox = new QVBoxLayout(this);
	QHBox *hbox = new QHBox(this);
	base_vbox->addWidget(hbox);

	listview = new QListView(hbox);
	listview->addColumn("Type");
	listview->addColumn("Path");
	listview->setSorting(-1);   // sort disabled
	listview->setAllColumnsShowFocus(true);

	QVBox *vbox = new QVBox(hbox);
	vbox->setFixedWidth(BUTTON_WIDTH*2);
//	vbox->setSizePolicy(QSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum));

//	base_vbox->addWidget(listview);

	add_button = new QPushButton(QString::fromUtf8("追加"),vbox);
//	add_button->setMinimumWidth(40);
	modify_button = new QPushButton(QString::fromUtf8("編集"),vbox);
//	modify_button->setMinimumWidth(40);
	del_button = new QPushButton(QString::fromUtf8("削除"),vbox);
//	del_button->setMinimumWidth(40);
	QHBox *updown_hbox = new QHBox(vbox);
	updown_hbox->setFixedWidth(BUTTON_WIDTH*2);
	updown_hbox->setSizePolicy(QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed));
	up_button = new QPushButton(QString::fromUtf8("↑"),updown_hbox);
	up_button->setFixedWidth(BUTTON_WIDTH);
	down_button = new QPushButton(QString::fromUtf8("↓"),updown_hbox);
	down_button->setFixedWidth(BUTTON_WIDTH);

	new QLabel(vbox);

#ifndef QWS
	QHBox *bhbox = new QHBox(this);
	QPushButton *cancel = new QPushButton("Cancel", bhbox);
	QPushButton *ok = new QPushButton("OK", bhbox);
	ok->setDefault(true);

	connect(cancel, SIGNAL(clicked()),
			this, SLOT(reject()));
	connect(ok, SIGNAL(clicked()),
			this, SLOT(accept()));

	base_vbox->addWidget(bhbox);
#endif

	connect(add_button, SIGNAL(clicked()),
			this, SLOT(add_click()));
	connect(modify_button, SIGNAL(clicked()),
			this, SLOT(modify_click()));
	connect(del_button, SIGNAL(clicked()),
			this, SLOT(del_click()));
	connect(up_button, SIGNAL(clicked()),
			this, SLOT(up_click()));
	connect(down_button, SIGNAL(clicked()),
			this, SLOT(down_click()));
}

MapPathDialog::~MapPathDialog()
{
}

void MapPathDialog::add_click()
{
	FileDialog dlg(this,
				   QString::fromUtf8("イメージファイル/ディレクトリの追加"),
				   FileDialog::Open, "", true);

	int ret = dlg.exec();
	if(ret == QDialog::Accepted){
		addPath(dlg.getFileName());
	}
}

void MapPathDialog::modify_click()
{
	QListViewItem *cur = listview->currentItem();
	if(!cur)
		return;

	FileDialog dlg(this,
				   QString::fromUtf8("イメージファイル/ディレクトリの追加"),
				   FileDialog::Open, cur->text(1), true);

	int ret = dlg.exec();
	if(ret == QDialog::Accepted){
		QString path = dlg.getFileName();
		cur->setText(0, getType(path));
		cur->setText(1, path);
	}
}

void MapPathDialog::del_click()
{
	QListViewItem *cur = listview->currentItem();
	if(!cur)
		return;

	listview->takeItem(cur);
	delete cur;

	if(listview->childCount() < MAXMAPPATH)
		add_button->setEnabled(true);
}

void MapPathDialog::up_click()
{
	QListViewItem *cur = listview->currentItem();
	if(!cur)
		return;

	QListViewItem *above = cur->itemAbove();
	if(!above)
		return;

	above = above->itemAbove();
	if(above){
		cur->moveItem(above);
	}else{
		listview->takeItem(cur);
		listview->insertItem(cur);
		listview->setSelected(cur, true);
	}
}

void MapPathDialog::down_click()
{
	QListViewItem *cur = listview->currentItem();
	if(!cur)
		return;

	QListViewItem *below = cur->itemBelow();
	if(!below)
		return;

	cur->moveItem(below);
}

QString MapPathDialog::getType(QString const &path)
{
	QString type = "???";

	if(path.right(1) == "/"){
		type = "DIR";
	} else if(path.right(4).lower() == ".iso"){
		type = "ISO";
	} else if(path.right(4).lower() == ".rom"){
		type = "ROM";
	}

	return type;
}

void MapPathDialog::addPath(QString const &path)
{
	QString type;

	type = getType(path);

	QListViewItem *item = listview->firstChild();
	while(item && item->itemBelow())
		item = item->itemBelow();

	if(item)
		new QListViewItem(listview, item, type, path);
	else
		new QListViewItem(listview, type, path);

	if(listview->childCount() >= MAXMAPPATH)
		add_button->setEnabled(false);
}

void MapPathDialog::loadConfig()
{
	QGConf *conf = (QGConf *)lookupObject(this, "QGConf", "config");
	QString str = "MapPath%0", path;
	int i;

	listview->clear();

	for(i = MAXMAPPATH - 1; i >= 0; i--){
		path = conf->readEntry(str.arg(i), "");

		if(path != ""){
			addPath(path);
		}
	}
}

void MapPathDialog::saveConfig()
{
	QGConf *conf = (QGConf *)lookupObject(this, "QGConf", "config");
	QString str = "MapPath%0", path;
	QListViewItem *item = listview->firstChild();
	int i = 0;

	for(i = 0; i < MAXMAPPATH; i++){
		conf->removeEntry(str.arg(i));
	}

	i=0;
	while(item){
		conf->writeEntry(str.arg(i), item->text(1));
		item = item->itemBelow();
		i++;
	}
}

void MapPathDialog::accept()
{
	if(listview->childCount() > 0){
		QDialog::accept();
		return;
	}

	QMessageBox::information(this, QString::fromUtf8("通知"),
							 QString::fromUtf8("地図のパスは最低1つ必要です。\n"
											   "デフォルトパスを追加します。"));
	addPath(DEFAULT_MAP_PATH);
}
