/*
  poi_mod_dialog.cpp
  Copyright (C) 2008-2009 Tomohiro Masubuchi <bucchi@e-ml.jp>

  This file is part of qgmap.

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; If not, see <http://www.gnu.org/licenses/>.
*/

#include "qgmap.h"
#include "poi_mod_dialog.h"
#include <qstring.h>
#include <qlabel.h>
#include <qvbox.h>
#include <qhbox.h>
#include <qpushbutton.h>
#include <qsizepolicy.h>
#include <qlayout.h>
#include <qlineedit.h>
#include <qpixmap.h>
#include <qmessagebox.h>
#include <qfileinfo.h>
#include <qcombobox.h>
#include <qspinbox.h>
#include <qlistbox.h>

#include "pos.h"
#include "filedialog.h"
#include "poi_db.h"
#include "poi_icon_dialog.h"

#define DIALOG_WIDTH  400
#ifdef QWS
#define DIALOG_HEIGHT (FRAME_HEIGHT*2+4)
#else
#define DIALOG_HEIGHT (FRAME_HEIGHT*2+40)
#endif

#define FRAME_HEIGHT 58

PoiModDialog::PoiModDialog( QWidget* parent, const char* name,
								bool modal, WFlags fl)
	: QDialog( parent, name, modal, fl )
{
    resize( QSize(DIALOG_WIDTH, 1).expandedTo(minimumSizeHint()) );

	setCaption(QString::fromUtf8("カスタムPOI"));

//	setFont(QFont("lcfont", 16));

	QVBoxLayout *vbox = new QVBoxLayout(this, 0, 3);

	QHBox *icon_hbox = new QHBox(this);
	new QLabel(QString::fromUtf8("Zoom:"), icon_hbox);
	zoom_spin = new QSpinBox(0, 19, 1, icon_hbox);

	QLabel *dummy = new QLabel(icon_hbox);
	dummy->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding));

	new QLabel(QString::fromUtf8("Icon:"), icon_hbox);
	icon_combo = new QComboBox(icon_hbox);
	QListBox *combo_list = icon_combo->listBox();
	combo_list->setMinimumHeight(150);
	QPushButton *add_button =
		new QPushButton(QString::fromUtf8("追加"), icon_hbox);
	QPushButton *del_button =
		new QPushButton(QString::fromUtf8("削除"), icon_hbox);
	vbox->addWidget(icon_hbox);

	QHBox *name_hbox = new QHBox(this);
	new QLabel(QString::fromUtf8("名前:"), name_hbox);
	name_edit = new QLineEdit(name_hbox);
	vbox->addWidget(name_hbox);

	connect(add_button, SIGNAL(clicked()), this, SLOT(press_add_button()));
	connect(del_button, SIGNAL(clicked()), this, SLOT(press_del_button()));

	updateIcon();

#ifndef QWS
	QHBox *op_hbox = new QHBox(this);
	QPushButton *cancel = new QPushButton("Cancel", op_hbox);
	QPushButton *ok = new QPushButton("OK", op_hbox);
	ok->setDefault(true);
	vbox->addWidget(op_hbox);

	connect(cancel, SIGNAL(clicked()),
			this, SLOT(reject()));
	connect(ok, SIGNAL(clicked()),
			this, SLOT(accept()));
#endif
}

PoiModDialog::~PoiModDialog()
{
}

void PoiModDialog::setId(int id)
{
	PoiDb db(DB_POI_PATH);
	PoiDbFile *file = db.queryPoiFile(-1, id);

	if (!file)
		return;

	if (file->next()) {
		name_edit->setText(file->getName());
	}

	delete file;
}

void PoiModDialog::setZoom(int zoom)
{
	zoom_spin->setValue(zoom);
}

int PoiModDialog::getIconId()
{
	return icon_id_table[icon_combo->currentItem()];
}

int PoiModDialog::getZoom()
{
	return zoom_spin->text().toInt();
}

QString PoiModDialog::getName()
{
	return name_edit->text();
}

void PoiModDialog::press_add_button()
{
	PoiIconDialog dlg(this);

	if (dlg.exec() == QDialog::Accepted) {
		int id = dlg.getIconId();
		updateIcon();

		unsigned int i;
		for (i = 0; i < icon_id_table.size(); i++) {
			if (icon_id_table[i] == id) {
				icon_combo->setCurrentItem(i);
				break;
			}
		}
	}
}

void PoiModDialog::press_del_button()
{
	if (QMessageBox::warning(this, QString::fromUtf8("削除の確認"),
							 QString::fromUtf8("アイコンを削除してよろしいですか?"),
							 QMessageBox::Cancel | QMessageBox::Default,
							 QMessageBox::Ok | QMessageBox::Escape) != QMessageBox::Ok) {
		return;
	}

	PoiDb db(DB_POI_PATH);
	int ret = db.delIcon(getIconId());
//	printf("ret = %d\n", ret);
	updateIcon();
}

void PoiModDialog::updateIcon()
{
	PoiDb db(DB_POI_PATH);

	PoiDbIcon *icon = db.queryPoiIcon();
	if (!icon)
		return;

	icon_combo->clear();
	if (!icon->next()) {
		delete icon;
		return;
	}

//	printf("icon->count=%d\n", icon->count());
	icon_id_table.resize(icon->count());

	int i = 0;
	do {
		QPixmap pixmap = QPixmap(icon->getIconData());
		if (!pixmap.isNull()) {
			icon_combo->insertItem(pixmap);
			icon_id_table[i] = icon->getId();
			i++;
		}
	} while(icon->next());

	delete icon;
}
